@@ -18,7 +18,7 @@
 #ifndef __GRABKEY_H
 #define __GRABKEY_H
 
-extern void grab_keys (Display * dpy, Window win);
+extern void grab_keys (Display * dpy);
 
 extern void get_offending_modifiers (Display * dpy);
 
