@@ -21,6 +21,7 @@
 #include <X11/Xlib.h>
 #include <signal.h>
 #include <unistd.h>
+#include <string.h>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -112,7 +113,7 @@ inner_main (int argc, char **argv)
       exit (0);
     }
 
-  grab_keys (d, 0);
+  grab_keys (d);
 
   /* This: for restarting reading the RC file if get a HUP signal */
   signal (SIGHUP, catch_HUP_signal);
@@ -152,6 +153,7 @@ static Display *
 start (char *display)
 {
   Display *d;
+  int screen;
 
   d = XOpenDisplay (display);
   if (!d)
@@ -161,11 +163,15 @@ start (char *display)
 and export or setenv it!\n");
       exit (1);
     }
+
+
   XAllowEvents (d, AsyncBoth, CurrentTime);
 
-  XSelectInput (d, DefaultRootWindow (d),
-		KeyPressMask | KeyReleaseMask | PointerMotionMask);
-
+  for (screen = 0; screen < ScreenCount (d); screen++)
+    {
+      XSelectInput (d, RootWindow (d, screen),
+		    KeyPressMask | KeyReleaseMask | PointerMotionMask);
+    }
 
   return (d);
 }
@@ -175,14 +181,43 @@ and export or setenv it!\n");
 void
 end_it_all (Display * d)
 {
-  XUngrabKey (d, AnyKey, AnyModifier, DefaultRootWindow (d));
-  XUngrabButton (d, AnyButton, AnyModifier, DefaultRootWindow (d));
+  int screen;
+
+  for (screen = 0; screen < ScreenCount (d); screen++)
+    {
+      XUngrabKey (d, AnyKey, AnyModifier, RootWindow (d, screen));
+      XUngrabButton (d, AnyButton, AnyModifier, RootWindow (d, screen));
+    }
   close_keys ();
   XCloseDisplay (d);
 }
 
 
 
+static void
+adjust_display (XAnyEvent * xany)
+{
+  char* envstr = malloc (strlen(DisplayString(xany->display)) + 8 + 1);
+  XWindowAttributes attr;
+  char* ptr;
+  char buf[16];
+
+  sprintf (envstr, "DISPLAY=%s", DisplayString(xany->display));
+
+  XGetWindowAttributes (xany->display, xany->window,  &attr);
+
+  if (verbose)
+    printf ("got screen %d for window %x\n", XScreenNumberOfScreen(attr.screen), (unsigned int)xany->window );
+
+  ptr = strchr (strchr (envstr, ':'), '.');
+  if (ptr)
+    *ptr = '\0';
+
+  snprintf (buf, sizeof(buf), ".%i", XScreenNumberOfScreen(attr.screen));
+  strcat (envstr, buf);
+
+  putenv (envstr);
+}
 
 
 
@@ -218,6 +253,7 @@ event_loop (Display * d)
 		      && e.xkey.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -228,6 +264,7 @@ event_loop (Display * d)
 		      && e.xkey.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -252,6 +289,7 @@ event_loop (Display * d)
 		      && e.xkey.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -262,6 +300,7 @@ event_loop (Display * d)
 		      && e.xkey.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -288,6 +327,7 @@ event_loop (Display * d)
 		      && e.xbutton.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -314,6 +354,7 @@ event_loop (Display * d)
 		      && e.xbutton.state == keys[i].modifier)
 		    {
 		      print_key (d, &keys[i]);
+		      adjust_display(&e.xany);
 		      start_command_key (&keys[i]);
 		    }
 		}
@@ -357,13 +398,17 @@ static void
 catch_HUP_signal (int sig)
 {
   int min, max;
+  int screen;
 
   XDisplayKeycodes (local_dpy, &min, &max);
 
   if (verbose)
     printf ("Catch HUP signal -> reread RC file\n");
 
-  XUngrabKey (local_dpy, AnyKey, AnyModifier, DefaultRootWindow (local_dpy));
+  for (screen = 0; screen < ScreenCount (local_dpy); screen++)
+    {
+      XUngrabKey (local_dpy, AnyKey, AnyModifier, RootWindow (local_dpy, screen));
+    }
   close_keys ();
 
 #ifdef GUILE_FLAG
@@ -379,7 +424,7 @@ catch_HUP_signal (int sig)
     }
 #endif
 
-  grab_keys (local_dpy, 0);
+  grab_keys (local_dpy);
 }
 
 
